@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+import threading
+from math import floor
 from . import Solar, NineStar
 from .util import ShouXingUtil, LunarUtil
 
@@ -12,7 +14,7 @@ class LunarYear:
 
     YUN = ("七", "八", "九", "一", "二", "三", "四", "五", "六")
 
-    __LEAP_11 = (75, 94, 170, 238, 265, 322, 389, 469, 553, 583, 610, 678, 735, 754, 773, 849, 887, 936, 1050, 1069, 1126, 1145, 1164, 1183, 1259, 1278, 1308, 1373, 1403, 1441, 1460, 1498, 1555, 1593, 1612, 1631, 1642, 2033, 2128, 2147, 2242, 2614, 2728, 2910, 3062, 3244, 3339, 3616, 3711, 3730, 3825, 4007, 4159, 4197, 4322, 4341, 4379, 4417, 4531, 4599, 4694, 4713, 4789, 4808, 4971, 5085, 5104, 5161, 5180, 5199, 5294, 5305, 5476, 5677, 5696, 5772, 5791, 5848, 5886, 6049, 6068, 6144, 6163, 6258, 6402, 6440, 6497, 6516, 6630, 6641, 6660, 6679, 6736, 6774, 6850, 6869, 6899, 6918, 6994, 7013, 7032, 7051, 7070, 7089, 7108, 7127, 7146, 7222, 7271, 7290, 7309, 7366, 7385, 7404, 7442, 7461, 7480, 7491, 7499, 7594, 7624, 7643, 7662, 7681, 7719, 7738, 7814, 7863, 7882, 7901, 7939, 7958, 7977, 7996,
+    __LEAP_11 = (75, 94, 170, 265, 322, 398, 469, 553, 583, 610, 678, 735, 754, 773, 849, 887, 936, 1050, 1069, 1126, 1145, 1164, 1183, 1259, 1278, 1308, 1373, 1403, 1441, 1460, 1498, 1555, 1593, 1612, 1631, 1642, 2033, 2128, 2147, 2242, 2614, 2728, 2910, 3062, 3244, 3339, 3616, 3711, 3730, 3825, 4007, 4159, 4197, 4322, 4341, 4379, 4417, 4531, 4599, 4694, 4713, 4789, 4808, 4971, 5085, 5104, 5161, 5180, 5199, 5294, 5305, 5476, 5677, 5696, 5772, 5791, 5848, 5886, 6049, 6068, 6144, 6163, 6258, 6402, 6440, 6497, 6516, 6630, 6641, 6660, 6679, 6736, 6774, 6850, 6869, 6899, 6918, 6994, 7013, 7032, 7051, 7070, 7089, 7108, 7127, 7146, 7222, 7271, 7290, 7309, 7366, 7385, 7404, 7442, 7461, 7480, 7491, 7499, 7594, 7624, 7643, 7662, 7681, 7719, 7738, 7814, 7863, 7882, 7901, 7939, 7958, 7977, 7996,
                  8034, 8053, 8072, 8091, 8121, 8159, 8186, 8216, 8235, 8254, 8273, 8311, 8330, 8341, 8349, 8368, 8444, 8463, 8474, 8493, 8531, 8569, 8588, 8626, 8664, 8683, 8694, 8702, 8713, 8721, 8751, 8789, 8808, 8816, 8827, 8846, 8884, 8903, 8922, 8941, 8971, 9036, 9066, 9085, 9104, 9123, 9142, 9161, 9180, 9199, 9218, 9256, 9294, 9313, 9324, 9343, 9362, 9381, 9419, 9438, 9476, 9514, 9533, 9544, 9552, 9563, 9571, 9582, 9601, 9639, 9658, 9666, 9677, 9696, 9734, 9753, 9772, 9791, 9802, 9821, 9886, 9897, 9916, 9935, 9954, 9973, 9992)
 
     __LEAP_12 = (37, 56, 113, 132, 151, 189, 208, 227, 246, 284, 303, 341, 360, 379, 417, 436, 458, 477, 496, 515, 534, 572, 591, 629, 648, 667, 697, 716, 792, 811, 830, 868, 906, 925, 944, 963, 982, 1001, 1020, 1039, 1058, 1088, 1153, 1202, 1221, 1240, 1297, 1335, 1392, 1411, 1422, 1430, 1517, 1525, 1536, 1574, 3358, 3472, 3806, 3988, 4751, 4941, 5066, 5123, 5275, 5343, 5438, 5457, 5495, 5533, 5552, 5715, 5810, 5829, 5905, 5924, 6421, 6535, 6793, 6812, 6888, 6907, 7002, 7184, 7260, 7279, 7374, 7556, 7746, 7757, 7776, 7833, 7852, 7871, 7966, 8015, 8110, 8129, 8148, 8224, 8243, 8338, 8406, 8425, 8482, 8501, 8520, 8558, 8596, 8607, 8615, 8645, 8740, 8778, 8835, 8865, 8930, 8960, 8979, 8998, 9017, 9055, 9074, 9093, 9112, 9150, 9188, 9237, 9275, 9332, 9351, 9370, 9408, 9427, 9446, 9457, 9465,
@@ -20,7 +22,10 @@ class LunarYear:
 
     __LEAP = {}
 
-    __CACHE = {}
+    __CACHE_YEAR = None
+
+    __lock = threading.Lock()
+
 
     for i in range(0, len(__LEAP_11)):
         __LEAP[__LEAP_11[i]] = 13
@@ -44,80 +49,100 @@ class LunarYear:
 
     @staticmethod
     def fromYear(lunar_year):
-        if lunar_year not in LunarYear.__CACHE:
-            obj = LunarYear(lunar_year)
-            LunarYear.__CACHE[lunar_year] = obj
+        LunarYear.__lock.acquire()
+        if LunarYear.__CACHE_YEAR is None or LunarYear.__CACHE_YEAR.getYear() != lunar_year:
+            y = LunarYear(lunar_year)
+            LunarYear.__CACHE_YEAR = y
         else:
-            obj = LunarYear.__CACHE[lunar_year]
-        return obj
+            y = LunarYear.__CACHE_YEAR
+        LunarYear.__lock.release()
+        return y
 
     def compute(self):
         from . import Lunar, Solar, LunarMonth
-        # 节气(中午12点)，长度27
+        # 节气
         jq = []
-        # 合朔，即每月初一(中午12点)，长度16
+        # 合朔，即每月初一
         hs = []
         # 每月天数，长度15
         day_counts = []
+        # 月份
+        months = []
 
         current_year = self.__year
-        year = current_year - 2000
+        jd = floor((current_year - 2000) * 365.2422 + 180)
+        # 355是2000.12冬至，得到较靠近jd的冬至估计值
+        w = floor((jd - 355 + 183) / 365.2422) * 365.2422 + 355
+        if ShouXingUtil.calcQi(w) > jd:
+            w -= 365.2422
+        # 25个节气时刻(北京时间)，从冬至开始到下一个冬至以后
+        for i in range(0, 26):
+            jq.append(ShouXingUtil.calcQi(w + 15.2184 * i))
 
-        # 从上年的大雪到下年的立春
+        # 从上年的大雪到下年的立春 精确的节气
         for i in range(0, len(Lunar.JIE_QI_IN_USE)):
-            # 精确的节气
-            t = 36525 * ShouXingUtil.saLonT((year + (17 + i) * 15.0 / 360) * ShouXingUtil.PI_2)
-            t += ShouXingUtil.ONE_THIRD - ShouXingUtil.dtT(t)
-            self.__jieQiJulianDays.append(t + Solar.J2000)
-            # 按中午12点算的节气
-            if 0 < i < 26:
-                jq.append(round(t))
-        # 冬至前的初一
+            if i == 0:
+                jd = ShouXingUtil.qiAccurate2(jq[0] - 15.2184)
+            elif i <= 26:
+                jd = ShouXingUtil.qiAccurate2(jq[i - 1])
+            else:
+                jd = ShouXingUtil.qiAccurate2(jq[25] + 15.2184 * (i - 26))
+            self.__jieQiJulianDays.append(jd + Solar.J2000)
+
+        # 冬至前的初一，今年"首朔"的日月黄经差w
         w = ShouXingUtil.calcShuo(jq[0])
         if w > jq[0]:
-            if current_year != 41 and current_year != 193 and current_year != 288 and current_year != 345 and current_year != 918 and current_year != 1013:
-                w -= 29.5306
+            w -= 29.53
         # 递推每月初一
         for i in range(0, 16):
             hs.append(ShouXingUtil.calcShuo(w + 29.5306 * i))
-        # 每月天数
+        # 每月
         for i in range(0, 15):
             day_counts.append(int(hs[i + 1] - hs[i]))
+            months.append(i)
 
         prev_year = current_year - 1
-        leap_year = -1
-        leap_index = -1
+        leap_index = 16
 
-        if current_year in LunarYear.__LEAP:
-            leap_index = LunarYear.__LEAP[current_year]
-            leap_year = current_year
-        else:
-            if prev_year in LunarYear.__LEAP:
-                leap_index = LunarYear.__LEAP[prev_year] - 12
-                leap_year = prev_year
-            else:
-                if hs[13] <= jq[24]:
-                    i = 1
-                    while hs[i + 1] > jq[2 * i] and i < 13:
-                        i += 1
-                    leap_year = current_year
-                    leap_index = i
+        if current_year in LunarYear.__LEAP_11:
+            leap_index = 13
+        elif current_year in LunarYear.__LEAP_12:
+            leap_index = 14
+        elif hs[13] <= jq[24]:
+            i = 1
+            while hs[i + 1] > jq[2 * i] and i < 13:
+                i += 1
+            leap_index = i
+        for j in range(leap_index, 15):
+            months[j] -= 1
+        ymc = [11, 12, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
 
+        fm = -1
+        index = -1
         y = prev_year
-        m = 11
-        index = m
         for i in range(0, 15):
-            cm = m
-            if y == leap_year and i == leap_index:
-                cm = -m
-            self.__months.append(LunarMonth(y, cm, day_counts[i], hs[i] + Solar.J2000, index))
-            if y != leap_year or i + 1 != leap_index:
-                m += 1
-            index += 1
-            if m == 13:
-                m = 1
-                index = 1
+            dm = hs[i] + Solar.J2000
+            v2 = months[i]
+            mc = ymc[v2 % 12]
+            if 1724360 <= dm < 1729794:
+                mc = ymc[(v2 + 1) % 12]
+            elif 1807724 <= dm < 1808699:
+                mc = ymc[(v2 + 1) % 12]
+            elif dm == 1729794 or dm == 1808699:
+                mc = 12
+            if fm == -1:
+                fm = mc
+                index = mc
+            if mc < fm:
                 y += 1
+                index = 1
+            fm = mc
+            if i == leap_index:
+                mc = -mc
+            elif dm == 1729794 or dm == 1808699:
+                mc = -11
+            self.__months.append(LunarMonth(y, mc, day_counts[i], dm, index))
+            index += 1
 
     def getYear(self):
         return self.__year
